@@ -5,6 +5,7 @@ import logging
 import random
 import time
 from collections import ChainMap, abc, defaultdict, namedtuple
+from contextvars import ContextVar
 from functools import partial
 from itertools import chain
 from typing import Any, Collection, List, Mapping, Optional, Tuple, Union
@@ -17,6 +18,9 @@ from .base import (
     AbortedException,
     IncompleteExecutionError,
     Items,
+    Operation,
+    PlotArgs,
+    Plottable,
     aslist,
     astuple,
     first_solid,
@@ -39,7 +43,6 @@ from .network import (
     yield_node_names,
     yield_ops,
 )
-from .base import Operation, PlotArgs, Plottable
 
 #: If this logger is *eventually* DEBUG-enabled,
 #: the string-representation of network-objects (network, plan, solution)
@@ -283,14 +286,14 @@ class _OpTask:
     This intermediate class is needed to solve pickling issue with process executor.
     """
 
-    __slots__ = ("op", "sol", "result", "solid")
+    __slots__ = ("op", "sol", "solid", "result")
     logname = __name__
 
-    def __init__(self, op, sol, solid):
+    def __init__(self, op, sol, solid, result=UNSET):
         self.op = op
         self.sol = sol
-        self.result = UNSET
         self.solid = solid
+        self.result = result
 
     def marshalled(self):
         import dill
@@ -302,7 +305,13 @@ class _OpTask:
             self.result = None
             log = logging.getLogger(self.logname)
             log.debug("+++ (%s) Executing %s...", self.solid, self)
-            self.result = self.op.compute(self.sol)
+            token = task_context.set(self)
+            try:
+                ## Not really needed ...
+                # self.result = copy_context().run(self.op.compute, self.sol)
+                self.result = self.op.compute(self.sol)
+            finally:
+                task_context.reset(token)
 
         return self.result
 
@@ -316,6 +325,11 @@ class _OpTask:
         return f"OpTask({self.op}, sol_keys={sol_items!r})"
 
 
+#: Populated with the :class:`_OpTask` for the currently executing operation.
+#: It does not work for :term:`parallel execution`.
+task_context: ContextVar[_OpTask] = ContextVar("task_context")
+
+
 def _do_task(task):
     """
     Un-dill the *simpler* :class:`_OpTask` & Dill the results, to pass through pool-processes.
@@ -657,7 +671,9 @@ class ExecutionPlan(
             else:
                 raise AssertionError(f"Unrecognized instruction.{step}")
 
-    def execute(self, named_inputs, outputs=None, *, name="") -> Solution:
+    def execute(
+        self, named_inputs, outputs=None, *, name="", solution_class=None
+    ) -> Solution:
         """
         :param named_inputs:
             A mapping of names --> values that must contain at least
@@ -667,6 +683,10 @@ class ExecutionPlan(
         :param outputs:
             If not None, they are just checked if possible, based on :attr:`provides`,
             and scream if not.
+        :param name:
+            name of the pipeline used for logging
+        :param solution_class:
+            a custom solution factory to use
 
         :return:
             The :term:`solution` which contains the results of each operation executed
@@ -704,7 +724,11 @@ class ExecutionPlan(
             #
             evict = self.asked_outs and not is_skip_evictions()
             # Note: clone and keep original `inputs` in the 1st chained-map.
-            solution = Solution(
+
+            if solution_class is None:
+                solution_class = Solution
+
+            solution = solution_class(
                 self,
                 {k: v for k, v in named_inputs.items() if k in self.dag.nodes}
                 if evict
