@@ -5,6 +5,11 @@
 
 The `needs` and `provides` annotated with *modifiers* designate, for instance,
 :term:`optional <optionals>` function arguments, or "ghost" :term:`sideffects`.
+
+.. note::
+    This module (along with :mod:`.op` & :mod:`.pipeline`) is what client code needs
+    to define pipelines *on import time* without incurring a heavy price
+    (~7ms on a 2019 fast PC)
 """
 import enum
 from typing import Optional, Tuple, Union
@@ -13,19 +18,19 @@ from typing import Optional, Tuple, Union
 #: Arguments-presence patterns for :class:`_Modifier` constructor.
 #: Combinations missing raise errors.
 _modifier_cstor_matrix = {
-# (7, kw, opt, sfxed, sfx): (STR, REPR) OR None
+# (7, kw, opt, sfxed, sfx): (STR, REPR, FUNC) OR None
 70000: None,
-71000: (       "%(dep)s",                  "'%(dep)s'(%(kw)s)"           ),
-71100: (       "%(dep)s",                  "'%(dep)s'(?%(kw)s)"          ),
-70200: (       "%(dep)s",                  "'%(dep)s'(*)"                ),
-70300: (       "%(dep)s",                  "'%(dep)s'(+)"                ),
-70010: ( "sfx: '%(dep)s'",            "sfx: '%(dep)s'"                   ),
-70110: ( "sfx: '%(dep)s'",         "sfx(?): '%(dep)s'"                    ),
-70011: ("sfxed('%(dep)s', %(sfx)s)", "sfxed('%(dep)s', %(sfx)s)"         ),
-71011: ("sfxed('%(dep)s', %(sfx)s)", "sfxed('%(dep)s'(%(kw)s), %(sfx)s)" ),
-71111: ("sfxed('%(dep)s', %(sfx)s)", "sfxed('%(dep)s'(?%(kw)s), %(sfx)s)"),
-70211: ("sfxed('%(dep)s', %(sfx)s)", "sfxed('%(dep)s'(*), %(sfx)s)"      ),
-70311: ("sfxed('%(dep)s', %(sfx)s)", "sfxed('%(dep)s'(+), %(sfx)s)"      ),
+71000: (       "%(dep)s",                  "'%(dep)s'(%(kw)s)"           , "mapped"),
+71100: (       "%(dep)s",                  "'%(dep)s'(?%(kw)s)"          , "optional"),
+70200: (       "%(dep)s",                  "'%(dep)s'(*)"                , "vararg"),
+70300: (       "%(dep)s",                  "'%(dep)s'(+)"                , "varargs"),
+70010: (  "sfx('%(dep)s')",            "sfx('%(dep)s')"                  , "sfx"),
+70110: (  "sfx('%(dep)s')",            "sfx('%(dep)s'(?))"               , "sfx"),
+70011: ("sfxed('%(dep)s', %(sfx)s)", "sfxed('%(dep)s', %(sfx)s)"         , "sfxed"),
+71011: ("sfxed('%(dep)s', %(sfx)s)", "sfxed('%(dep)s'(%(kw)s), %(sfx)s)" , "sfxed"),
+71111: ("sfxed('%(dep)s', %(sfx)s)", "sfxed('%(dep)s'(?%(kw)s), %(sfx)s)", "sfxed"),
+70211: ("sfxed('%(dep)s', %(sfx)s)", "sfxed('%(dep)s'(*), %(sfx)s)"      , "sfxed_vararg"),
+70311: ("sfxed('%(dep)s', %(sfx)s)", "sfxed('%(dep)s'(+), %(sfx)s)"      , "sfxed_varargs"),
 }
 # fmt: on
 
@@ -58,9 +63,9 @@ class _Modifier(str):
 
     - the factory functions :func:`.mapped`, :func:`optional` etc,
     - the predicates :func:`is_optional()`, :func:`is_pure_sfx()` predicates, etc,
-    - and the :func:`dep_rename()`, :func:`dep_strip()` conversion functions
+    - and the :func:`dep_renamed()`, :func:`dep_stripped()` conversion functions
 
-    respectively.
+    respectively, or at least :func:`_modifier()`.
 
     .. Note::
         User code better call :func:`_modifier()` factory which may return a plain string
@@ -74,6 +79,7 @@ class _Modifier(str):
         "sideffected",
         "sfx_list",
         "_repr",
+        "_func",
     )
 
     #: Map my name in `needs` into this kw-argument of the function.
@@ -83,7 +89,8 @@ class _Modifier(str):
     #: :func:`is_optional()` returns it.
     #: All regulars are `mapped`.
     optional: _Optionals
-    #: the pure-sideffect string or the existing :term:`sideffected` dependency
+    #: Has value only for sideffects: the pure-sideffect string or
+    #: the existing :term:`sideffected` dependency.
     sideffected: str
     #: At least one name(s) denoting the :term:`sideffects` modification(s) on
     #: the :term:`sideffected`, performed/required by the operation.
@@ -95,9 +102,11 @@ class _Modifier(str):
     sfx_list: Tuple[Union[str, None]]
     #: pre-calculated representation
     _repr: str
+    #: needed to reconstruct cstor code in :attr:`cmd`
+    _func: str
 
     def __new__(
-        cls, name, fn_kwarg, optional: _Optionals, sideffected, sfx_list, _repr,
+        cls, name, fn_kwarg, optional: _Optionals, sideffected, sfx_list, _repr, _func
     ) -> "_Modifier":
         """Warning, returns None! """
         ## sanity checks & preprocessing
@@ -126,12 +135,27 @@ class _Modifier(str):
         obj.sideffected = sideffected
         obj.sfx_list = sfx_list
         obj._repr = _repr
+        obj._func = _func
 
         return obj
 
     def __repr__(self):
         return self._repr
 
+    @property
+    def cmd(self):
+        """the code to reproduce it"""
+        dep = self.sideffected or str(self)
+        items = [f"'{dep}'"]
+        if self.sfx_list:
+            items.append(", ".join(f"'{i}'" for i in self.sfx_list))
+        if self.fn_kwarg and self.fn_kwarg != dep:
+            fn_kwarg = f"'{self.fn_kwarg}'"
+            items.append(f"fn_kwarg={fn_kwarg}" if self.sfx_list else fn_kwarg)
+        if self.optional == _Optionals.keyword and self._func != "optional":
+            items.append(f"optional=1" if self.sfx_list else "1")
+        return f"{self._func}({', '.join(items)})"
+
     def __getnewargs__(self):
         return (
             str(self),
@@ -140,6 +164,7 @@ class _Modifier(str):
             self.sideffected,
             self.sfx_list,
             self._repr,
+            self._func,
         )
 
     def _withset(
@@ -154,7 +179,7 @@ class _Modifier(str):
         Make a new modifier with changes -- handle with care.
 
         :return:
-             Delegates to :func:`_dep`, so returns a plain string if no args left.
+             Delegates to :func:`_modifier`, so returns a plain string if no args left.
         """
         kw = {
             k: getattr(self, k) if v is ... else v
@@ -166,10 +191,10 @@ class _Modifier(str):
         if name is ...:
             name = self.sideffected or str(self)
 
-        return _dep(name=name, **kw)
+        return _modifier(name=name, **kw)
 
 
-def _dep(
+def _modifier(
     name, fn_kwarg=None, optional: _Optionals = None, sideffected=None, sfx_list=(),
 ):
     """
@@ -183,16 +208,17 @@ def _dep(
         # Make a plain string instead.
         return str(name)
 
-    str_fmt, repr_fmt = formats
+    str_fmt, repr_fmt, func = formats
     fmt_args = {
         "dep": name,
+        # avoid a sole ``?>`` symbol on pure optionals
         "kw": f">'{fn_kwarg}'" if fn_kwarg != name else ">" if not optional else "",
         "sfx": ", ".join(f"'{i}'" for i in sfx_list),
     }
     name = str_fmt % fmt_args
     _repr = repr_fmt % fmt_args
 
-    return _Modifier(name, fn_kwarg, optional, sideffected, sfx_list, _repr)
+    return _Modifier(name, fn_kwarg, optional, sideffected, sfx_list, _repr, func)
 
 
 def mapped(name: str, fn_kwarg: str = None):
@@ -236,7 +262,7 @@ def mapped(name: str, fn_kwarg: str = None):
 
         >>> graph = compose('mygraph', myadd)
         >>> graph
-        NetworkOperation('mygraph', needs=['a', 'name-in-inputs'], provides=['sum'], x1 ops: myadd)
+        Pipeline('mygraph', needs=['a', 'name-in-inputs'], provides=['sum'], x1 ops: myadd)
 
         >>> sol = graph.compute({"a": 5, "name-in-inputs": 4})['sum']
         >>> sol
@@ -245,7 +271,7 @@ def mapped(name: str, fn_kwarg: str = None):
         .. graphtik::
     """
     # Must pass a truthy `fn_kwarg` bc cstor cannot not know its mapped.
-    return _dep(name, fn_kwarg=fn_kwarg or name)
+    return _modifier(name, fn_kwarg=fn_kwarg or name)
 
 
 def optional(name: str, fn_kwarg: str = None):
@@ -277,7 +303,7 @@ def optional(name: str, fn_kwarg: str = None):
 
         >>> graph = compose('mygraph', myadd)
         >>> graph
-        NetworkOperation('mygraph',
+        Pipeline('mygraph',
                          needs=['a', 'b'(?)],
                          provides=['sum'],
                          x1 ops: myadd)
@@ -303,7 +329,7 @@ def optional(name: str, fn_kwarg: str = None):
 
     """
     # Must pass a truthy `fn_kwarg` as cstor-matrix requires.
-    return _dep(name, fn_kwarg=fn_kwarg or name, optional=_Optionals.keyword)
+    return _modifier(name, fn_kwarg=fn_kwarg or name, optional=_Optionals.keyword)
 
 
 def vararg(name: str):
@@ -347,7 +373,7 @@ def vararg(name: str):
         {'a': 5, 'sum': 5}
 
     """
-    return _dep(name, optional=_Optionals.vararg)
+    return _modifier(name, optional=_Optionals.vararg)
 
 
 def varargs(name: str):
@@ -370,7 +396,7 @@ def varargs(name: str):
         ...     provides='sum')(enlist)
         ... )
         >>> graph
-        NetworkOperation('mygraph',
+        Pipeline('mygraph',
                          needs=['a', 'b'(?)],
                          provides=['sum'],
                          x1 ops: enlist)
@@ -409,7 +435,7 @@ def varargs(name: str):
 
     .. varargs-mistake-end
     """
-    return _dep(name, optional=_Optionals.varargs)
+    return _modifier(name, optional=_Optionals.varargs)
 
 
 def sfx(name, optional: bool = None):
@@ -440,7 +466,7 @@ def sfx(name, optional: bool = None):
         >>> from graphtik import sfx
 
         >>> sfx("price[sales_df]")
-        sfx: 'price[sales_df]'
+        sfx('price[sales_df]')
 
     **Example:**
 
@@ -462,9 +488,9 @@ def sfx(name, optional: bool = None):
         ...         provides="body")(lambda: "TaDa!")
         ... )
         >>> graph
-        NetworkOperation('strip ease',
-                         needs=[sfx: 'lights off'],
-                         provides=[sfx: 'lights off', 'body'],
+        Pipeline('strip ease',
+                         needs=[sfx('lights off')],
+                         provides=[sfx('lights off'), 'body'],
                          x2 ops: close_the_lights, undress)
 
         >>> sol = graph()
@@ -484,7 +510,7 @@ def sfx(name, optional: bool = None):
         .. graphtik::
 
     """
-    return _dep(
+    return _modifier(
         name, optional=_Optionals.keyword if optional else None, sideffected=name,
     )
 
@@ -544,17 +570,17 @@ def sfxed(
     To view all internal :term:`dependencies <dependency>`, enable DEBUG
     in :term:`configurations`:
 
-        >>> from graphtik import debug_enabled
+        >>> from graphtik.config import debug_enabled
 
         >>> with debug_enabled(True):
         ...     finalize_prices
         FunctionalOperation(name='finalize_prices',
                             needs=[sfxed('ORDER', 'Prices'), sfxed('ORDER', 'VAT')],
                             op_needs=[sfxed('ORDER', 'Prices'), sfxed('ORDER', 'VAT')],
-                            fn_needs=['ORDER'],
+                            _fn_needs=['ORDER'],
                             provides=[sfxed('ORDER', 'Totals')],
                             op_provides=[sfxed('ORDER', 'Totals')],
-                            fn_provides=['ORDER'],
+                            _fn_provides=['ORDER'],
                             fn='finalize_prices')
 
     Notice that declaring a single *sideffected* with many items in `sfx_list`,
@@ -586,7 +612,7 @@ def sfxed(
 
 
     """
-    return _dep(
+    return _modifier(
         dependency,
         optional=_Optionals.keyword if optional else None,
         fn_kwarg=dependency if optional and not fn_kwarg else fn_kwarg,
@@ -597,7 +623,7 @@ def sfxed(
 
 def sfxed_vararg(dependency: str, sfx0: str, *sfx_list: str):
     """Like :func:`sideffected` + :func:`vararg`. """
-    return _dep(
+    return _modifier(
         dependency,
         optional=_Optionals.vararg,
         sideffected=dependency,
@@ -607,7 +633,7 @@ def sfxed_vararg(dependency: str, sfx0: str, *sfx_list: str):
 
 def sfxed_varargs(dependency: str, sfx0: str, *sfx_list: str):
     """Like :func:`sideffected` + :func:`varargs`. """
-    return _dep(
+    return _modifier(
         dependency,
         optional=_Optionals.varargs,
         sideffected=dependency,
@@ -674,11 +700,24 @@ def is_sfxed(dep) -> bool:
     return getattr(dep, "sideffected", None) and getattr(dep, "sfx_list", None)
 
 
+def dependency(dep):
+    """
+    For non-sideffects, it coincides with str(), otherwise,
+    the the pure-sideffect string or the existing :term:`sideffected` dependency
+    stored in :attr:`sideffected`.
+    """
+    return str(dep) if is_sfx(dep) else dep.sideffected
+
+
 def dep_renamed(dep, ren):
     """
     Renames `dep` as `ren` or call `ren`` (if callable) to decide its name,
 
     preserving any :func:`mapped` to old-name.
+
+    For :term:`sideffected` it renames the dependency (not the *sfx-list*) --
+    you have to do it that manually with a custom renamer-function, if ever
+    the need arise.
     """
     if callable(ren):
         renamer = ren
@@ -686,9 +725,11 @@ def dep_renamed(dep, ren):
         renamer = lambda n: ren
 
     if isinstance(dep, _Modifier):
-        old_name = dep.sideffected if is_sfx(dep) else str(dep)
-        new_name = renamer(old_name)
-        dep = dep._withset(name=new_name)
+        if is_sfx(dep):
+            new_name = renamer(dep.sideffected)
+            dep = dep._withset(name=new_name, sideffected=new_name)
+        else:
+            dep = dep._withset(name=renamer(str(dep)))
     else:  # plain string
         dep = renamer(dep)
 
