@@ -1,42 +1,47 @@
 # Copyright 2016, Yahoo Inc.
 # Licensed under the terms of the Apache License, Version 2.0. See the LICENSE file associated with the project for terms.
 """
-Define :term:`operation` :term:`dependency` & :term:`modifier`\\s, and match/zip inputs/outputs .
+Define :term:`operation` & :term:`dependency` and match/zip inputs/outputs during :term:`execution`.
 
-... but not net-ops to break the cycle.
+.. note::
+    This module (along with :mod:`.modifiers` & :mod:`.pipeline`) is what client code needs
+    to define pipelines *on import time* without incurring a heavy price
+    (<5ms on a 2019 fast PC)
 """
 
-import abc
 import itertools as itt
 import logging
 import textwrap
 from collections import abc as cabc
 from functools import wraps
-from typing import Any, Callable, Collection, List, Mapping, Tuple, Union
+from typing import (
+    Any,
+    Callable,
+    Collection,
+    Hashable,
+    List,
+    Mapping,
+    Tuple,
+)
 
 from boltons.setutils import IndexedSet as iset
 
 from .base import (
-    NO_RESULT,
     UNSET,
     Items,
     MultiValueError,
-    PlotArgs,
+    Operation,
     Plottable,
+    PlotArgs,
+    Token,
     aslist,
     astuple,
+    first_solid,
     func_name,
     jetsam,
 )
-from .config import (
-    first_solid,
-    is_debug,
-    is_endure_operations,
-    is_marshal_tasks,
-    is_parallel_tasks,
-    is_reschedule_operations,
-)
 from .modifiers import (
+    dep_renamed,
     dep_singularized,
     dep_stripped,
     is_mapped,
@@ -46,10 +51,21 @@ from .modifiers import (
     is_sfxed,
     is_vararg,
     is_varargs,
+    optional,
 )
 
 log = logging.getLogger(__name__)
 
+#: A special return value for the function of a :term:`reschedule` operation
+#: signifying that it did not produce any result at all (including :term:`sideffects`),
+#: otherwise, it would have been a single result, ``None``.
+#: Usefull for rescheduled who want to cancel their single result
+#: witout being delcared as :term:`returns dictionary`.
+NO_RESULT = Token("NO_RESULT")
+#: Like :data:`NO_RESULT` but does not cancel any :term;`sideffects`
+#: declared as provides.
+NO_RESULT_BUT_SFX = Token("NO_RESULT_BUT_SFX")
+
 
 def as_renames(i, argname):
     """
@@ -86,18 +102,21 @@ def as_renames(i, argname):
 
 
 def reparse_operation_data(
-    name, needs, provides
-) -> Tuple[cabc.Hashable, cabc.Collection, cabc.Collection]:
+    name, needs, provides, aliases=()
+) -> Tuple[
+    Hashable, Collection[str], Collection[str], Collection[Tuple[str, str]],
+]:
     """
     Validate & reparse operation data as lists.
 
     :return:
-        name, needs, provides,
+        name, needs, provides, aliases
 
     As a separate function to be reused by client building operations,
     to detect errors early.
     """
-
+    # if name is not None and not name or not isinstance(name, cabc.Hashable):
+    #     raise TypeError(f"Operation `name` must be a truthy hashable, got: {name}")
     if not isinstance(name, cabc.Hashable):
         raise TypeError(f"Operation `name` must be hashable, got: {name}")
 
@@ -111,36 +130,45 @@ def reparse_operation_data(
     if not all(isinstance(i, str) for i in provides):
         raise TypeError(f"All `provides` must be str, got: {provides!r}")
 
-    return name, needs, provides
+    aliases = as_renames(aliases, "aliases")
+    if aliases:
+        if not all(
+            src and isinstance(src, str) and dst and isinstance(dst, str)
+            for src, dst in aliases
+        ):
+            raise TypeError(f"All `aliases` must be non-empty str, got: {aliases!r}")
+        if any(1 for src, dst in aliases if dst in provides):
+            bad = ", ".join(
+                f"{src} -> {dst}" for src, dst in aliases if dst in provides
+            )
+            raise ValueError(
+                f"The `aliases` [{bad}] clash with existing provides in {list(provides)}!"
+            )
 
+        alias_src = iset(src for src, _dst in aliases)
+        if not alias_src <= set(provides):
+            bad_alias_sources = alias_src - provides
+            bad_aliases = ", ".join(
+                f"{src!r}-->{dst!r}" for src, dst in aliases if src in bad_alias_sources
+            )
+            raise ValueError(
+                f"The `aliases` [{bad_aliases}] rename non-existent provides in {list(provides)}!"
+            )
+        sfx_aliases = [
+            f"{src} -> {dst}" for src, dst in aliases if is_sfx(src) or is_sfx(dst)
+        ]
+        if sfx_aliases:
+            raise ValueError(
+                f"The `aliases` must not contain `sideffects` {sfx_aliases}"
+                "\n  Simply add any extra `sideffects` in the `provides`."
+            )
 
-class Operation(abc.ABC):
-    """An abstract class representing an action with :meth:`.compute()`."""
-
-    @property
-    def __name__(self) -> str:
-        return self.name  # pylint: disable=no-member
-
-    @abc.abstractmethod
-    def compute(self, named_inputs, outputs=None):
-        """
-        Compute (optional) asked `outputs` for the given `named_inputs`.
-
-        It is called by :class:`.Network`.
-        End-users should simply call the operation with `named_inputs` as kwargs.
-
-        :param named_inputs:
-            the input values with which to feed the computation.
-        :returns list:
-            Should return a list values representing
-            the results of running the feed-forward computation on
-            ``inputs``.
-        """
+    return name, needs, provides, aliases
 
 
 def _spread_sideffects(
-    deps: cabc.Collection,
-) -> Tuple[cabc.Collection, cabc.Collection]:
+    deps: Collection[str],
+) -> Tuple[Collection[str], Collection[str]]:
     """
     Build fn/op dependencies from user ones by stripping or singularizing any :term:`sideffects`.
 
@@ -229,38 +257,14 @@ class FunctionalOperation(Operation, Plottable):
         if name is None and fn:
             name = func_name(fn, None, mod=0, fqdn=0, human=0, partials=1)
         ## Overwrite reparsed op-data.
-        name, needs, provides = reparse_operation_data(name, needs, provides)
+        name, needs, provides, aliases = reparse_operation_data(
+            name, needs, provides, aliases
+        )
 
         needs, _fn_needs = _spread_sideffects(needs)
         provides, _fn_provides = _spread_sideffects(provides)
         op_needs = iset(needs)
-
-        if aliases:
-            aliases = as_renames(aliases, "aliases")
-            if any(1 for src, dst in aliases if dst in provides):
-                bad = ", ".join(
-                    f"{src} -> {dst}" for src, dst in aliases if dst in provides
-                )
-                raise ValueError(
-                    f"The `aliases` ({bad}) clash with existing provides {list(provides)}!"
-                )
-
-            alias_src, alias_dst = list(zip(*aliases))
-            if not set(alias_src) <= set(provides):
-                raise ValueError(
-                    f"The `aliases` for {list(alias_src)} rename {list(iset(alias_src) - provides)}"
-                    f", not found in provides {list(provides)}!"
-                )
-            sfx_aliases = [
-                f"{src} -> {dst}" for src, dst in aliases if is_sfx(src) or is_sfx(dst)
-            ]
-            if sfx_aliases:
-                raise ValueError(
-                    f"The `aliases` must not contain `sideffects` {sfx_aliases}"
-                    "\n  Simply add any extra `sideffects` in the `provides`."
-                )
-        else:
-            alias_dst = ()
+        alias_dst = aliases and tuple(dst for _src, dst in aliases)
         op_provides = iset(itt.chain(provides, alias_dst))
 
         #: The :term:`operation`'s underlying function.
@@ -324,7 +328,7 @@ class FunctionalOperation(Operation, Plottable):
         #: but it is easier for that function to simply call :meth:`.set_results_by_name()`.
         self.returns_dict = returns_dict
         #: Added as-is into NetworkX graph, and you may filter operations by
-        #: :meth:`.NetworkOperation.withset()`.
+        #: :meth:`.Pipeline.withset()`.
         #: Also plot-rendering affected if they match `Graphviz` properties,
         #: unless they start with underscore(``_``).
         self.node_props = node_props
@@ -341,32 +345,39 @@ class FunctionalOperation(Operation, Plottable):
         """
         Display more informative names for the Operation class
         """
-        needs = aslist(self.needs, "needs")
-        provides = aslist(self.provides, "provides")
-        aliases = aslist(self.aliases, "aliases")
-        aliases = f", aliases={aliases!r}" if aliases else ""
+        from .config import (
+            is_debug,
+            is_endure_operations,
+            is_marshal_tasks,
+            is_parallel_tasks,
+            is_reschedule_operations,
+            reset_abort,
+        )
+
+        dep_names = (
+            "needs op_needs _fn_needs provides op_provides _fn_provides aliases"
+            if is_debug()
+            else "needs provides aliases"
+        ).split()
+        deps = [(i, getattr(self, i)) for i in dep_names]
         fn_name = self.fn and func_name(self.fn, None, mod=0, fqdn=0, human=0)
-        nprops = f", x{len(self.node_props)}props" if self.node_props else ""
+        returns_dict_marker = self.returns_dict and "{}" or ""
+        items = [
+            f"name={self.name!r}",
+            *(f"{n}={aslist(d, n)}" for n, d in deps if d),
+            f"fn{returns_dict_marker}={fn_name!r}",
+        ]
+        if self.node_props:
+            items.append(f"x{len(self.node_props)}props")
+
         resched = (
             "?" if first_solid(self.rescheduled, is_reschedule_operations()) else ""
         )
         endured = "!" if first_solid(self.endured, is_endure_operations()) else ""
         parallel = "|" if first_solid(self.parallel, is_parallel_tasks()) else ""
         marshalled = "&" if first_solid(self.marshalled, is_marshal_tasks()) else ""
-        returns_dict_marker = self.returns_dict and "{}" or ""
 
-        if is_debug():
-            debug_needs = (
-                f", op_needs={list(self.op_needs)}, fn_needs={list(self._fn_needs)}"
-            )
-            debug_provides = f", op_provides={list(self.op_provides)}, fn_provides={list(self._fn_provides)}"
-        else:
-            debug_needs = debug_provides = ""
-        return (
-            f"FunctionalOperation{endured}{resched}{parallel}{marshalled}(name={self.name!r}, "
-            f"needs={needs!r}{debug_needs}, provides={provides!r}{debug_provides}{aliases}, "
-            f"fn{returns_dict_marker}={fn_name!r}{nprops})"
-        )
+        return f"FunctionalOperation{endured}{resched}{parallel}{marshalled}({', '.join(items)})"
 
     @property
     def deps(self) -> Mapping[str, Collection]:
@@ -375,6 +386,7 @@ class FunctionalOperation(Operation, Plottable):
 
         if not DEBUG, all deps are converted into lists, ready to be printed.
         """
+        from .config import is_debug
 
         return {
             k: v if is_debug() else list(v)
@@ -391,8 +403,98 @@ class FunctionalOperation(Operation, Plottable):
             )
         }
 
-    def withset(self, fn: Callable = None, **kw,) -> "FunctionalOperation":
-        """Make a *clone* with the some values replaced. """
+    def withset(
+        self,
+        fn: Callable = ...,
+        name=...,
+        needs: Items = ...,
+        provides: Items = ...,
+        aliases: Mapping = ...,
+        *,
+        rescheduled=...,
+        endured=...,
+        parallel=...,
+        marshalled=...,
+        returns_dict=...,
+        node_props: Mapping = ...,
+        renamer=None,
+    ) -> "FunctionalOperation":
+        """
+        Make a *clone* with the some values replaced, or operation and dependencies renamed.
+
+        if `renamer` given, it is applied on top (and afterwards) any other changed
+        values, for operation-name, needs, provides & any aliases.
+
+        :param renamer:
+            - if a dictionary, it renames any operations & data named as keys
+              into the respective values by feeding them into :func:.dep_renamed()`,
+              so values may be single-input callables themselves.
+            - if it is a :func:`.callable`, it is given a :class:`.RenArgs` instance
+              to decide the node's name.
+
+            The callable may return a *str* for the new-name, or any other false
+            value to leave node named as is.
+
+            .. Attention::
+                The callable SHOULD wish to preserve any :term:`modifier` on dependencies,
+                and use :func:`.dep_renamed()` if a callable is given.
+
+        :return:
+            a clone operation with changed/renamed values asked
+
+        :raise:
+            - (ValueError, TypeError): all cstor validation errors
+            - ValueError: if a `renamer` dict contains a non-string and non-callable
+              value
+
+
+        **Examples**
+
+            >>> from graphtik import sfx
+
+            >>> op = operation(str, "foo", needs="a",
+            ...     provides=["b", sfx("c")],
+            ...     aliases={"b": "B-aliased"})
+            >>> op.withset(renamer={"foo": "BAR",
+            ...                     'a': "A",
+            ...                     'b': "B",
+            ...                     sfx('c'): "cc",
+            ...                     "B-aliased": "new.B-aliased"})
+            FunctionalOperation(name='BAR',
+                                needs=['A'],
+                                provides=['B', sfx('cc')],
+                                aliases=[('B', 'new.B-aliased')],
+                                fn='str')
+
+        - Notice that ``'c'`` rename change the "sideffect name, without the destination name
+          being an ``sfx()`` modifier (but source name must match the sfx-specifier).
+        - Notice that the source of aliases from ``b-->B`` is handled implicitely
+          from the respective rename on the `provides`.
+
+        But usually a callable is more practical, like the one below renaming
+        only data names:
+
+            >>> op.withset(renamer=lambda ren_args:
+            ...            dep_renamed(ren_args.name, lambda n: f"parent.{n}")
+            ...            if ren_args.typ != 'op' else
+            ...            False)
+            FunctionalOperation(name='foo',
+                                needs=['parent.a'],
+                                provides=['parent.b', sfx('parent.c')],
+                                aliases=[('parent.b', 'parent.B-aliased')],
+                                fn='str')
+
+
+        Notice the double use of lambdas with :func:`dep_renamed()` -- an equivalent
+        rename callback would be::
+
+            dep_renamed(ren_args.name, f"parent.{dependency(ren_args.name)}")
+        """
+        kw = {
+            k: v
+            for k, v in locals().items()
+            if v is not ... and k not in "self renamer".split()
+        }
         ## Exclude calculated dep-fields.
         #
         me = {
@@ -400,10 +502,12 @@ class FunctionalOperation(Operation, Plottable):
             for k, v in vars(self).items()
             if not k.startswith("_") and not k.startswith("op_")
         }
-        if fn:
-            me["fn"] = fn
-        me.update(kw)
-        return FunctionalOperation(**me)
+        kw = {**me, **kw}
+
+        if renamer:
+            self._rename_graph_names(kw, renamer)
+
+        return FunctionalOperation(**kw)
 
     def _prepare_match_inputs_error(
         self,
@@ -412,6 +516,8 @@ class FunctionalOperation(Operation, Plottable):
         varargs_bad: List,
         named_inputs: Mapping,
     ) -> ValueError:
+        from .config import is_debug
+
         errors = [
             f"Need({n}) failed due to: {type(nex).__name__}({nex})"
             for n, nex in enumerate(exceptions, 1)
@@ -487,6 +593,8 @@ class FunctionalOperation(Operation, Plottable):
 
     def _zip_results_with_provides(self, results) -> dict:
         """Zip results with expected "real" (without sideffects) `provides`."""
+        from .config import is_reschedule_operations
+
         fn_expected: iset = self._fn_provides
         rescheduled = first_solid(is_reschedule_operations(), self.rescheduled)
         if self.returns_dict:
@@ -540,12 +648,16 @@ class FunctionalOperation(Operation, Plottable):
                         f"-{len(missmatched)} provides({list(fn_expected)})!\n  {self}"
                     )
 
-        elif results == NO_RESULT and rescheduled:
-            # Cancel also any SFX.
-            results = {p: False for p in set(self.provides) if is_sfx(p)}
+        elif results in (NO_RESULT, NO_RESULT_BUT_SFX) and rescheduled:
+            results = (
+                {}
+                if results == NO_RESULT_BUT_SFX
+                # Cancel also any SFX.
+                else {p: False for p in set(self.provides) if is_sfx(p)}
+            )
 
         elif not fn_expected:  # All provides were sideffects?
-            if results and results != NO_RESULT:
+            if results and results not in (NO_RESULT, NO_RESULT_BUT_SFX):
                 ## Do not scream,
                 #  it is common to call a function for its sideffects,
                 # which happens to return an irrelevant value.
@@ -559,7 +671,7 @@ class FunctionalOperation(Operation, Plottable):
         else:  # Handle result sequence: no-result, single-item, many
             nexpected = len(fn_expected)
 
-            if results == NO_RESULT:
+            if results in (NO_RESULT, NO_RESULT_BUT_SFX):
                 results = ()
                 ngot = 0
 
@@ -618,9 +730,9 @@ class FunctionalOperation(Operation, Plottable):
 
     def compute(self, named_inputs=None, outputs: Items = None) -> dict:
         try:
-            if self.fn is None:
+            if self.fn is None or not self.name:
                 raise ValueError(
-                    f"Operation was not yet provided with a callable `fn`!"
+                    f"Operation must have a callable `fn` and a non-empty `name`:\n  {self}"
                 )
             assert self.name is not None, self
             if named_inputs is None:
@@ -773,7 +885,7 @@ def operation(
         (i.e. the returned output-values are not zipped with `provides`)
     :param node_props:
         Added as-is into NetworkX graph, and you may filter operations by
-        :meth:`.NetworkOperation.withset()`.
+        :meth:`.Pipeline.withset()`.
         Also plot-rendering affected if they match `Graphviz` properties.,
         unless they start with underscore(``_``)
 
@@ -801,13 +913,14 @@ def operation(
 
         >>> op = op.withset(needs=['a', 'b'])
         >>> op
-        FunctionalOperation(name=None, needs=['a', 'b'], provides=[], fn=None)
+        FunctionalOperation(name=None, needs=['a', 'b'], fn=None)
 
     If you call an operation with `fn` un-initialized, it will scream:
 
         >>> op.compute({"a":1, "b": 2})
         Traceback (most recent call last):
-        ValueError: Operation was not yet provided with a callable `fn`!
+        ValueError: Operation must have a callable `fn` and a non-empty `name`:
+          FunctionalOperation(name=None, needs=['a', 'b'], fn=None)
 
     You may keep calling ``withset()`` until a valid operation instance is returned,
     and compute it:
@@ -827,7 +940,8 @@ def operation(
 
     if "fn" in kw:
         # Either used as a "naked" decorator (without any arguments)
-        # or not used as decorator at all (manually called and passed in `fn`) .
+        # or not used as decorator at all (manually called and passed in `fn`).
+        # Don't bother returning a decorator.
         return op
 
     @wraps(op.withset)
@@ -839,17 +953,3 @@ def operation(
     decorator.withset = op.withset
 
     return decorator
-
-
-class NULL_OP(FunctionalOperation):
-    """
-    Eliminates same-named operations added later during term:`operation merging`.
-
-    :seealso: :ref:`operation-merging`
-    """
-
-    def __init__(self, name):
-        super().__init__(name=name)
-
-    def compute(self, *args, **kw):
-        raise AssertionError("Should have been eliminated!")
