@@ -8,6 +8,7 @@ import re
 import sys
 import time
 import types
+from collections import namedtuple
 from functools import partial
 from itertools import cycle
 from multiprocessing import Pool, cpu_count
@@ -16,13 +17,14 @@ from multiprocessing import get_context
 from operator import add, floordiv, mul, sub
 from pprint import pprint
 from textwrap import dedent
+from time import sleep
 from typing import Tuple
 from unittest.mock import MagicMock
 
 import pytest
 
 from graphtik import network
-from graphtik.base import AbortedException, IncompleteExecutionError
+from graphtik.base import AbortedException, IncompleteExecutionError, Operation
 from graphtik.config import (
     abort_run,
     debug_enabled,
@@ -34,8 +36,7 @@ from graphtik.config import (
     operations_reschedullled,
     tasks_marshalled,
 )
-from graphtik.execution import Solution
-from graphtik.base import Operation
+from graphtik.execution import Solution, _OpTask, task_context
 from graphtik.modifiers import dep_renamed, optional, sfx, sfxed, vararg
 from graphtik.op import NO_RESULT, NO_RESULT_BUT_SFX, operation
 from graphtik.pipeline import NULL_OP, Pipeline, compose
@@ -49,10 +50,13 @@ _thread = pytest.mark.thread
 _parallel = pytest.mark.parallel
 _marshal = pytest.mark.marshal
 
+_ExeParams = namedtuple("_ExeParams", "parallel, proc, marshal")
+_exe_params = _ExeParams(None, None, None)
+
 
 @pytest.fixture(
     params=[
-        # PARALLEL?, Proc/Thread?, Marshalled?
+        # PARALLEL?, Thread/Proc?, Marshalled?
         (None, None, None),
         pytest.param((1, 0, 0), marks=(_parallel, _thread)),
         pytest.param((1, 0, 1), marks=(_parallel, _thread, _marshal)),
@@ -70,7 +74,10 @@ _marshal = pytest.mark.marshal
 )
 def exemethod(request):
     """Returns (exemethod, marshal) combinations"""
+    global _exe_params
     parallel, proc_pool, marshal = request.param
+    _exe_params = _ExeParams(*request.param)
+
     nsharks = None  # number of pool swimmers....
 
     with tasks_marshalled(marshal):
@@ -118,6 +125,21 @@ def abspow(a, p):
     return c
 
 
+def test_serialize_pipeline(samplenet, ser_method):
+    def eq(pipe1, pipe2):
+        return pipe1.name == pipe2.name and pipe1.ops == pipe2.ops
+
+    assert eq(ser_method(samplenet), samplenet)
+
+
+def test_serialize_OpTask(ser_method):
+    def eq(o1, o2):
+        return all(getattr(o1, a) == getattr(o2, a) for a in _OpTask.__slots__)
+
+    ot = _OpTask(1, 2, 3, 4)
+    assert eq(ser_method(ot), ot)
+
+
 def test_solution_finalized():
     sol = Solution(MagicMock(), {})
 
@@ -253,6 +275,13 @@ def test_network_plan_execute():
     plan = net.compile(outputs=outputs, inputs=inputs.keys())
     sol = plan.execute(inputs)
     assert sol == exp
+
+    # test solution factory
+    sol = plan.execute(
+        inputs, solution_class=(lambda *args, **kw: Solution(*args, **kw))
+    )
+    assert sol == exp
+
     sol = plan.execute(inputs, outputs)
     assert sol == exp
 
@@ -278,6 +307,42 @@ def test_network_plan_execute():
     assert sol == exp
 
 
+def test_task_context(exemethod, request):
+    def check_task_context():
+        sleep(0.1)
+        assert task_context.get().op == next(iop), "Corrupted task-context"
+
+    n_ops = 10
+    pipe = compose(
+        "t",
+        *(
+            operation(check_task_context, f"op{i}", provides=f"{i}")
+            for i in range(n_ops)
+        ),
+        parallel=exemethod,
+    )
+    iop = iter(pipe.ops)
+
+    print(_exe_params, os.cpu_count())
+    err = None
+    if _exe_params.proc and _exe_params.marshal:
+        err = Exception("^Error sending result")
+    elif _exe_params.parallel and _exe_params.marshal:
+        err = AssertionError("^Corrupted task-context")
+    elif _exe_params.parallel and not os.environ.get("TRAVIS"):
+        # Travis has low parallelism and error does not surface
+        err = AssertionError("^Corrupted task-context")
+
+    if err:
+        with pytest.raises(type(err), match=str(err)):
+            pipe.compute()
+        raise pytest.xfail("Cannot marshal parallel processes with `task_context` :-(.")
+    else:
+        pipe.compute()
+        with pytest.raises(StopIteration):
+            next(iop)
+
+
 def test_compose_rename_dict(caplog):
     pip = compose(
         "t",
@@ -1612,7 +1677,9 @@ def test_rescheduling_NO_RESULT(exemethod):
         assert sol.scream_if_incomplete()
 
 
-@pytest.mark.xfail(reason="Spurious copied-reversed graphs, with dubious cause....")
+@pytest.mark.xfail(
+    reason="Spurious copied-reversed graphs in Travis, with dubious cause...."
+)
 def test_multithreading_plan_execution():
     # Compose the mul, sub, and abspow operations into a computation graph.
     # From Huygn's test-code given in yahoo/graphkit#31
@@ -1689,7 +1756,9 @@ def test_parallel_execution(exemethod):
 
 
 @pytest.mark.slow
-@pytest.mark.xfail(reason="Spurious copied-reversed graphs, with dubious cause....")
+@pytest.mark.xfail(
+    reason="Spurious copied-reversed graphs in Travis, with dubious cause...."
+)
 def test_multi_threading_computes():
     import random
 
